@@ -9,6 +9,7 @@ use clap::Parser;
 use fs_err as fs;
 use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
 use path_slash::PathExt;
+use which::{which, which_in};
 use xwin::util::ProgressTarget;
 
 use crate::common::{CargoOptions, XWinOptions};
@@ -241,6 +242,61 @@ impl Build {
             if target.contains("msvc") {
                 self.setup_msvc_crt(xwin_cache_dir.clone())?;
                 let env_target = target.to_lowercase().replace('-', "_");
+                let env_path = env::var("PATH").unwrap_or_default();
+                let mut env_paths: Vec<_> = env::split_paths(&env_path).collect();
+
+                let env_path = if cfg!(target_os = "macos") {
+                    let mut new_path = env_path;
+                    new_path.push_str(":/opt/homebrew/opt/llvm/bin");
+                    new_path.push_str(":/usr/local/opt/llvm/bin");
+                    new_path
+                } else {
+                    env_path
+                };
+                let cache_dir = xwin_cache_dir.parent().unwrap();
+
+                if which_in("clang-cl", Some(env_path.clone()), env::current_dir()?).is_err() {
+                    if let Ok(clang) = which("clang") {
+                        #[cfg(windows)]
+                        {
+                            let symlink = cache_dir.join("clang-cl.exe");
+                            if !symlink.exists() {
+                                std::os::windows::fs::symlink_file(clang, symlink)?;
+                            }
+                        }
+
+                        #[cfg(unix)]
+                        {
+                            let symlink = cache_dir.join("clang-cl");
+                            if !symlink.exists() {
+                                std::os::unix::fs::symlink(clang, symlink)?;
+                            }
+                        }
+                    }
+                }
+                if which_in("lld-link", Some(env_path), env::current_dir()?).is_err() {
+                    let bin_dir = rustc_target_bin_dir()?;
+                    let rust_lld = bin_dir.join("rust-lld");
+                    if rust_lld.exists() {
+                        #[cfg(windows)]
+                        {
+                            let symlink = cache_dir.join("lld-link.exe");
+                            if !symlink.exists() {
+                                std::os::windows::fs::symlink_file(rust_lld, symlink)?;
+                            }
+                        }
+
+                        #[cfg(unix)]
+                        {
+                            let symlink = cache_dir.join("lld-link");
+                            if !symlink.exists() {
+                                std::os::unix::fs::symlink(rust_lld, symlink)?;
+                            }
+                        }
+                    }
+                }
+                env_paths.push(cache_dir.to_path_buf());
+
                 build.env("TARGET_CC", format!("clang-cl --target={}", target));
                 build.env("TARGET_CXX", format!("clang-cl --target={}", target));
                 build.env(
@@ -253,6 +309,7 @@ impl Build {
                 );
                 build.env("TARGET_AR", "llvm-lib");
                 build.env(format!("AR_{}", env_target), "llvm-lib");
+
                 build.env(
                     format!("CARGO_TARGET_{}_LINKER", env_target.to_uppercase()),
                     "lld-link",
@@ -277,25 +334,25 @@ impl Build {
 
                 let mut rustflags = env::var_os("RUSTFLAGS").unwrap_or_default();
                 rustflags.push(format!(
-                    " -Lnative={dir}/crt/lib/{arch} -Lnative={dir}/sdk/lib/um/{arch} -Lnative={dir}/sdk/lib/ucrt/{arch}",
+                    " -C linker-flavor=lld-link -Lnative={dir}/crt/lib/{arch} -Lnative={dir}/sdk/lib/um/{arch} -Lnative={dir}/sdk/lib/ucrt/{arch}",
                     dir = xwin_cache_dir.display(),
                     arch = xwin_arch,
                 ));
                 build.env("RUSTFLAGS", rustflags);
 
                 #[cfg(target_os = "macos")]
-                if let Ok(path) = env::var("PATH") {
-                    let mut new_path = path.clone();
-                    if cfg!(target_arch = "x86_64") && !path.contains("/usr/local/opt/llvm/bin") {
-                        new_path.push_str(":/usr/local/opt/llvm/bin");
-                    } else if cfg!(target_arch = "aarch64")
-                        && !path.contains("/opt/homebrew/opt/llvm/bin")
-                    {
-                        new_path.push_str(":/opt/homebrew/opt/llvm/bin");
+                {
+                    let usr_llvm = "/usr/local/opt/llvm/bin".into();
+                    let opt_llvm = "/opt/homebrew/opt/llvm/bin".into();
+                    if cfg!(target_arch = "x86_64") && !env_paths.contains(&usr_llvm) {
+                        env_paths.push(usr_llvm);
+                    } else if cfg!(target_arch = "aarch64") && !env_paths.contains(&opt_llvm) {
+                        env_paths.push(opt_llvm);
                     }
-                    build.env("PATH", new_path);
                 }
 
+                build.env("PATH", env::join_paths(env_paths)?);
+
                 // CMake support
                 let cmake_toolchain = self.setup_cmake_toolchain(target, &xwin_cache_dir)?;
                 build
@@ -558,3 +615,13 @@ pub fn adjust_canonicalization(p: String) -> String {
         p
     }
 }
+
+fn rustc_target_bin_dir() -> Result<PathBuf> {
+    let output = Command::new("rustc")
+        .args(&["--print", "target-libdir"])
+        .output()?;
+    let stdout = String::from_utf8(output.stdout)?;
+    let lib_dir = Path::new(&stdout);
+    let bin_dir = lib_dir.parent().unwrap().join("bin");
+    Ok(bin_dir)
+}
